@@ -16,7 +16,7 @@
  **/
 
 var background_page = (function () {
-   
+
    // TODO extract based on logging settings
    var log = {
      info: function (msg) {
@@ -32,7 +32,6 @@ var background_page = (function () {
      }
    };
 
-
    // connection & init w/ plugin NPAPI layer
    var plugin = document.getElementById ('unityChromiumExtensionId');
    if ( ! plugin) {
@@ -139,8 +138,7 @@ var background_page = (function () {
     *
     */
    var matchesIntegrationScripts = function (plugin, url, repo, windowInfos, callback) {
-     function askForApplicationInstallation(plugin, appPackageName, url, installationCallback) {
-       var appDomain = plugin.application_repository_get_resolved_application_domain(repo, appPackageName);
+     function askForApplicationInstallation(plugin, appPackageName, appDomain, url, installationCallback) {
 
        if (plugin.permissions_get_domain_dontask(appDomain)) {
          log.info ("WebApp domain was found in the 'dont ask' list, won't install: " + appPackageName);
@@ -194,9 +192,11 @@ var background_page = (function () {
      var APPLICATION_STATUS_INSTALLED = 1;
      var APPLICATION_STATUS_UNRESOLVED = 2;
 
-     var formatInstalledAppInfo = function (name, src) {
+     var formatMatchedWebAppInfo = function (packageName, appName, appDomain, src) {
        return {
-         name: name,
+         packageName: packageName,
+         appName: appName,
+         appDomain: appDomain,
          content: src,
          requires: null,
          includes: null
@@ -209,21 +209,27 @@ var background_page = (function () {
       * 
       * @returns object with "installed" and "available" list properties
       */
-     var gatherApplicationsStatuses = function (names) {
+     var gatherApplicationsStatuses = function (packageNames) {
        var installed = [];
        var available = [];
-       for (var i = 0; i < names.length; ++i) {
-         var status = plugin.application_repository_get_resolved_application_status (repo, names[i]);
-         log.info ('A WebApp application has been found: ' + names[i] + ', status: ' + status);
+       for (var i = 0; i < packageNames.length; ++i) {
+         var packageName = packageNames[i];
+
+         var status = plugin.application_repository_get_resolved_application_status (repo, packageName);
+         var appName = plugin.application_repository_get_resolved_application_name (repo, packageName);
+         var appDomain = plugin.application_repository_get_resolved_application_domain (repo, packageName);
+
+         log.info ('A WebApp application has been found: ' + appName + ', status: ' + status);
+
          switch (status) {
            case APPLICATION_STATUS_INSTALLED:
              // we have found an installed application, use it directly
-             var src = plugin.application_repository_get_userscript_contents (repo, names[i]);
-             installed.push (formatInstalledAppInfo(names[i], src));
+             var src = plugin.application_repository_get_userscript_contents (repo, packageName);
+             installed.push (formatMatchedWebAppInfo(packageName, appName, appDomain, src));
              break;
            case APPLICATION_STATUS_AVAILABLE:
              // we have found an application that can apply and be installed
-             available.push (names[i]);
+             available.push (formatMatchedWebAppInfo(packageName, appName, appDomain));
              break;
          }
        }
@@ -233,113 +239,54 @@ var background_page = (function () {
        };
      };
 
-     var names = JSON.parse(plugin.application_repository_resolve_url_as_json(repo_, url));
-     if (null != names) {
-       var scripts = gatherApplicationsStatuses(names);
-       if (scripts && scripts.installed && scripts.installed.length > 0) {
-         callback (scripts.installed);
+     var packageNames = JSON.parse(plugin.application_repository_resolve_url_as_json(repo_, url));
+     if (null != packageNames) {
+       var appStatuses = gatherApplicationsStatuses(packageNames);
+
+       if (appStatuses && appStatuses.installed && appStatuses.installed.length > 0) {
+         var app = appStatuses.installed[0];
+         if (repo_install_request_stamps[app.packageName] == undefined
+             && ! plugin.permissions_get_domain_dontask(app.appDomain)
+             && ! plugin.permissions_get_domain_allowed(app.appDomain)) {
+           // requesting for install
+           addInfobarRequestCallbackFor (windowInfos.tabId,
+                                         function (result) {
+                                           if (result && result.integrate) {
+                                             plugin.application_repository_add_desktop_to_launcher(app.appName, app.appDomain);
+                                             plugin.permissions_allow_domain(app.appDomain);
+                                           }
+                                           else {
+                                             plugin.permissions_dontask_domain(app.appDomain);
+                                           }
+                                         },
+	                                 chrome.i18n.getMessage("integration_copy", [app.appName, app.appDomain]),
+                                         null);
+           chrome.infobars.show ({tabId: windowInfos.tabId, path: "infobar.html"});
+         }
          return;
        }
 
        // we should have at most one script for a given app
-       if (scripts && scripts.available && scripts.available.length > 0) {
-         var name = scripts.available[0];
+       if (appStatuses && appStatuses.available && appStatuses.available.length > 0) {
+         var app = appStatuses.available[0];
          askForApplicationInstallation (plugin,
-                                        name,
+                                        app.packageName,
+                                        app.appDomain,
                                         url,
                                         function (_repository, name, status, data) {
                                           log.info ('Application installed: ' + name + ', (status ' + status + ')');
+
                                           if (name !== undefined && status == APPLICATION_STATUS_INSTALLED) {
-                                            var src = plugin.application_repository_get_userscript_contents (repo_, name);
-                                            scripts.installed.push (formatInstalledAppInfo(name, src));
+                                            callback (true, app.packageName, app.appName, app.appDomain);
                                           }
-                                          callback (scripts.installed);
                                         }
                                        );
        }
-       else  {
-         callback (null);
-       }
+       else
+         callback (false);
      } // if (null != names) {
 
    }; // var matchesIntegrationScripts = function (plugin, url, repo, windowInfos, callback) {
-
-
-   /**
-    * Handles tab change & activation code.
-    * TODO(FIXME): better window tracking for active windows & tabs
-    *   -> there are some glitches when a given webapp tab is moved around:
-    *      1. open chromeless webapp w/ webapp (A)
-    *      2. open normal window webapp (B) & unrelated other tab in same window
-    *      3. undock (B) and dock it in (A): issues w/ activation (in laucnher)
-    *      4. close the "first webapp" (A) -> actiating the webapp icon now
-    *         refers back to first window, might be bc of BAMF window tracking
-    *
-    */
-   chrome.tabs.onActivated.addListener (
-     function (info) {
-       // TODO(FIXME): pretty bad ... assumed that all tabs are interested
-       chrome.windows.getAll (
-         {populate: true}
-	 , function (windows) {
-	   for (var winIdx in windows) {
-	     var tabs = windows[winIdx].tabs;
-	     for (var tabIdx in tabs) {
-	       var tabId = tabs[tabIdx].id;
-	       chrome.tabs.sendRequest (tabId
-					, {method: "on_tab_active_changed"
-					   , tabId: info.tabId
-					  }
-					, function (response) {});
-	     } // for (var tabIdx
-	   } // for (var winIdx
-	 });
-     }
-   );
-
-   // TODO move elsewhere
-   // TODO check that's it's fine (it should), since contrary to
-   //      FF this is being called on the same environment every time (background page)
-   function toDataURL (uri, callback, sx, sy, sw, sh) {
-     if (uri.match(/^\s{0,}file:/i))
-       return;
-     
-     if (uri.match(/^\s{0,}https:/i))
-       return;
-     
-     var self = this;
-     var canvas = document.createElementNS("http://www.w3.org/1999/xhtml", "canvas");
-     var img = new window.Image();
-     
-     img.onload = function() {
-       var width = img.width, height = img.height;
-
-       var hasExtraArgs = sx || sx === 0;
-       if (hasExtraArgs) {
-	 width = sw;
-	 height = sh;
-       }
-
-       var dx  = 0, dy = 0;
-
-       if (width > height)
-	 dy = width - height;
-       else
-	 dx = height - width;
-
-       canvas.width = width + dx;
-       canvas.height = height + dy;
-       
-       var ctx = canvas.getContext('2d');
-       if (hasExtraArgs)
-	 ctx.drawImage(img, sx, sy, sw, sh, dx / 2, dy / 2, width, height);
-       else
-	 ctx.drawImage(img, dx / 2, dy / 2);
-       
-       callback (true, canvas.toDataURL());
-     };
-     img.src = uri;
-   };
    
    
    ////////////////////////////////////////
@@ -352,79 +299,10 @@ var background_page = (function () {
     * Handles & responds to content script requests.
     * 
     */
+   var init_requested_stamps = {};
    var contentScriptsRequestHandler = function (request, sender, callback) {
      
      var handlers = {
-       get_unity_script: function (request, sender, callback) {
-	 
-         var withMatchedScripts = function (scripts) {
-	   var results = [];
-	   if (scripts) {
-             for (var i = 0; i < scripts.length; ++i) {
-               var result = {
-	         userscript: null
-	         , requires: null
-	       };
-               result.userscript = scripts[i].content;
-	       if (scripts[i].requires && scripts[i].requires.length !== 0) {
-	         result.requires = scripts[i].requires;
-	       }
-	       results.push(result);
-             }
-	   }
-	   callback (results);
-         };
-	 matchesIntegrationScripts (plugin,
-                                    request.url,
-                                    repo_,
-                                    {
-                                      windowId: sender.windowId,
-                                      tabId: sender.tab.id
-                                    },
-                                    withMatchedScripts
-                                   );
-       }
-       ,
-       add_integrationscript_for_tabid: function (request, sender, callback) {
-         
-         addIntegrationScriptForTab(sender.tab.id
-                                    , request.name
-                                    , request.domain
-                                    , request.interest_id);
-       }
-       ,
-       // TODO(alex-abreu): those two should be refactored and be cleaner
-       on_user_infobar_request_result: function (request, sender, callback) {
-	 
-	 invokeAndRemoveCallbackIfAnyFor (request.tabId, request);
-       }
-       ,
-       request_user_integration: function (request, sender, callback) {
-	 
-         // keep information about the context of the infobar request
-	 addInfobarRequestCallbackFor (sender.tab.id, callback, request.message, request.details);
-	 
-	 chrome.infobars.show ({tabId: sender.tab.id, path: "infobar.html"});
-       }
-       , 
-       take_snapshot: function (request, sender, callback) {
-	 
-	 chrome.tabs.captureVisibleTab (sender.tab.windowId
-					, null
-					, function (dataURL) {
-					  callback (dataURL);
-					});
-       }
-       , 
-       get_data_url_from: function (request, sender, callback) {
-	 
-	 toDataURL (request.url
-		    , function (flag, dataUrl) {
-		      
-		      callback ({flag: flag, dataUrl: dataUrl});
-		    });
-       }
-       , 
        get_extension_settings: function (request, sender, callback) {
 	 
 	 var settings = {
@@ -443,37 +321,55 @@ var background_page = (function () {
 	 
 	 callback (settings);
        }
-       , 
-       get_tab_info: function (request, sender, callback) {
+       ,
+       on_user_infobar_request_result: function (request, sender, callback) {
 	 
-	 callback ({tabId: sender.tab.id, windowId: sender.tab.windowId});
+	 invokeAndRemoveCallbackIfAnyFor (request.tabId, request);
        }
-       , 
-       kill_this_tab: function (request, sender, callback) {
-	 
-	 chrome.tabs.remove ([request.tabId], function () {});
-       }
-       , 
-       raise_this_tab: function (request, sender, callback) {
-	 
-	 // bring the window & the tab to the front
-	 chrome.windows.update (request.windowId
-				, {focused: true}
-				, function (window) {
-				  chrome.tabs.update (request.tabId
-						      , {active: true, highlighted: true});
-				});
-       }
-       , 
-       is_selected: function (request, sender, callback) {
-	 
-	 chrome.tabs.getCurrent (
-	   function (tab) {
-	     if (tab) {
-	       callback ({isSelected: tab.id === request.tabId});
-	     }
-	   }
-	 );
+       ,
+       init_requested: function (request, sender, callback) {
+         var name = request.options.name;
+         var domain = request.options.domain;
+         var iconUrl = request.options.iconUrl + '';
+         var homepage = request.options.homepage || request.options.url;
+
+         if (! domain || ! name || typeof(domain) !== 'string' || typeof(name) !== 'string')
+           return;
+
+         var islocal = request.options.protocol && request.options.protocol === 'file:';
+         var host = request.options.hostname || request.options.host;
+         if ( ! islocal && (! host || typeof(host) !== 'string'))
+           return;
+
+         if (-1 === homepage.indexOf(domain) || -1 === host.indexOf(domain)) {
+           log.error('Invalid call to init(), homepage, domain and host are not consistent');
+           return;
+         }
+         log.info('init_requested() called for ' + name + ' domain: ' + domain);
+
+         if (plugin.permissions_get_domain_dontask(domain) || plugin.permissions_get_domain_allowed(domain)) {
+           log.info('Domain already listed in allow or dont ask lists');
+           return;
+         }
+
+         var stampname = name + domain;
+         if (init_requested_stamps[stampname] == undefined) {
+           addInfobarRequestCallbackFor (sender.tab.id,
+                                         function (result) {
+                                           if (result && result.integrate) {
+                                             plugin.application_repository_generate_desktop_for_url_launch(name, domain, homepage, iconUrl);
+                                             plugin.application_repository_add_desktop_to_launcher(name, domain);
+                                             plugin.permissions_allow_domain(domain);
+                                           }
+                                           else {
+                                             plugin.permissions_dontask_domain(domain);
+                                           }
+                                         },
+	                                 chrome.i18n.getMessage("integration_copy", [name, domain]),
+                                         null);
+           init_requested_stamps[stampname] = true;
+           chrome.infobars.show ({tabId: sender.tab.id, path: "infobar.html"});
+         }
        }
      }; // handlers
      
@@ -505,56 +401,44 @@ var background_page = (function () {
        return true;
      }
      return false;
-   }; // var contentScriptsRequestHandler =
+   } // var contentScriptsRequestHandler =
 
    // Main event handler and communication link
    //   w/ content scripts
    chrome.runtime.onMessage.addListener (contentScriptsRequestHandler);
 
 
-   ///////////////////////////////////////////////
+   ///////////////////////////////////////////////////////////////////////
    // 
-   // window/tab management handling
-   //
-   ///////////////////////////////////////////////
+   // Window management related functions. In chromeless mode, we have specific
+   //  rules for tab management to make webapps feel more "native" than plain
+   //  web applications.
+   // 
+   ///////////////////////////////////////////////////////////////////////
 
-   // Every tab that's being integrated as a webapp (chromeless or not)
-   //  actually registers here. This allows tabs life cycle to be managed
-   //  a bit when removed, or in chromeless mode when moved around.
-   var integration_script_tabs = {};
-   var addIntegrationScriptForTab = function (tabId
-                                              , name
-                                              , domain
-                                              , interest_id) {
-     if (integration_script_tabs[tabId] !== undefined) {
-       log.info('Overriding tab information for existing tab: ' + tabId);
-     }
-     log.info ('Integrating tab with name: ' + name + ', for domain ' + domain);
-     integration_script_tabs[tabId] = {name: name
-                                       , domain: domain
-                                       , interest_id: interest_id};
+  var onTabChanged = function (tabId, windowId, url) {
+    var onInstalled = function (installed, packageName, appName, appDomain) { };
+    matchesIntegrationScripts (plugin,
+                               url,
+                               repo_,
+                               {
+                                 windowId: windowId,
+                                 tabId: tabId
+                               },
+                               onInstalled
+                              );
    };
-   var removeIntegrationScriptForTab = function (tabId) {
-     if (integration_script_tabs[tabId] !== undefined) {
-       integration_script_tabs[tabId] = undefined;
+   chrome.tabs.onCreated.addListener(
+     function(tab) {
+       if (tab && tab.url) {
+         onTabChanged(tab.id, tab.windowId, tab.url);
+       } // if (tab && tab.url) {
      }
-   };
-   chrome.tabs.onRemoved.addListener (
-     function (tabId, removeInfo) {
-       var info = integration_script_tabs[tabId];
-       removeIntegrationScriptForTab (tabId);
-       if (info !== undefined) {
-         log.info('Destroying interest: ' + info.name + ', ' + info.domain + ', ' + info.interest_id);
-         try {
-           plugin.service_destroy_interest_for_context (service
-                                                        , info.name
-                                                        , info.domain
-                                                        , info.interest_id
-                                                        , 1);
-         }
-         catch (e) {
-           log.error('Error while trying to destroy interest for tab ' + tabId + ', ' + e);
-         }
+   );
+   chrome.tabs.onUpdated.addListener (
+     function(tabId, changeInfo, tab) {
+       if (changeInfo && changeInfo.url) {
+         onTabChanged(tabId, tab.windowId, changeInfo.url);
        }
      }
    );
