@@ -8,19 +8,16 @@
    var aWindow = currentWindow;
    var plugin = null;
    var service = null;
-   var api = null;
-
-   var webkitNotificationUnityBackend = {
-     showNotification: function (iconUrl, title, body) {
-       try {
-         var notification = plugin.notification_new(title, body, null/*iconUrl*/);
-         plugin.notification_show(notification, null);
-         plugin.notification_destroy(notification);
+   var api = {
+       init: function (options) {
+	   options.url = window.location.href;
+	   options.hostname = window.location.hostname;
+	   options.host = window.location.host;
+	   options.protocol = window.location.protocol;
+	   chrome.runtime.sendMessage (
+	       {method: "init_requested", options: options}
+	       , function (response) {});
        }
-       catch (err) {
-         // TODO decipher/inject proper & general logging policy for user side of API
-       }
-     }
    };
 
    /**
@@ -40,7 +37,6 @@
 
      // 1. inject a piece of javascript proxying all the requests
      // to the unity webapps api
-     insertScriptIntoWebpage('webkitnotif-wrapper-builder.js');
      insertScriptIntoWebpage('unity-api-page-proxy-builder-gen.js');
      insertScriptIntoWebpage('unity-api-page-proxy.js');
 
@@ -114,10 +110,6 @@
        var reducetarget = api;
 
        // a bit hacky
-       if (funcnames[0] == 'webkitNotifications') {
-         funcnames.shift();
-         reducetarget = webkitNotificationUnityBackend;
-       }
        try {
          // Assumes that we are calling a 'callable' from a succession of objects
          funcnames.reduce (
@@ -154,65 +146,9 @@
                                , true);
    };
 
-   function initializePluginConnection (logger){
-     plugin = document.getElementById ("unityChromiumExtensionId");
-     if ( ! plugin) {
-       logger ('Could not bind to the scriptable NPAPI plugin');
-     }
-     else {
-       logger ("Got a plugin object: " + plugin);
-       
-       // initialize webkitnotification backend & fallback
-       try {
-         plugin.notification_init("chromium");
-       }
-       catch (err) {
-         logger ("Error while trying to initialize notify OSD NPAPI connection");
-         // decide to bail out
-         return;
-       }
-       UnityWebappsWebkitNotificationApiPageProxyBuilder(webkitNotificationUnityBackend, aWindow);
-
-       // initialize unity API binding
-       service = plugin.service_new ();
-       logger ("Service object created: " + service);
-
-       // global plugin initialization
-       // FIXME: there seem to be an issue that started w/ chromium 27+
-       //  where NPobnject v8 bindings are looked up in the wrong
-       //  JS world under some circumstances.
-       api = UnityGlobalPropertyInitializer.makeCallback (
-	   function() {
-	       return document.getElementById ("unityChromiumExtensionId");
-	   },
-	   service,
-	   logger)(aWindow);
-     }
-   }
-
-   initializePluginConnection (function (msg) {
-				 if (consoleService && consoleService.logStringMessage) {
-				   consoleService.logStringMessage ("Unity Webapps: " + msg);
-				 }
-			       });
-
    // handle the proxy side of the api which is being injected on the
    //  webpage
    injectApiProxy();
-   
-   var windowExternal = aWindow.external;
-   function unity() {
-   }
-   unity.prototype = {
-     __proto__: windowExternal,
-     getUnityObject: function (version) {
-       console.log ('getUnityObject called with version ' + version);
-       if (version === 1)
-         return api;
-       throw new Error("incorrect version");
-     }
-   };
-   aWindow.external = new unity();
  }
 )(window);
 
