@@ -1365,7 +1365,7 @@ unity_webapps_binding_music_player_set_track (NPP instance
   gchar * title = NULL;
   if (! is_null_or_void (args[3]))
     {
-      album = create_safe_string_for (&args[3]);
+      title = create_safe_string_for (&args[3]);
     }
 
   gchar * icon_url = NULL;
